package com.qmth.themis.business.cache;

import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.*;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;
import org.bouncycastle.util.Strings;

import java.util.Date;
import java.util.Objects;

/**
 * 考试记录缓存hash值操作
 *
 * @Description:
 * @Author: xiatian
 * @Date: 2020-07-29
 */
public class ExamRecordCacheUtil {

    private static TOeExamRecordService examRecordService = SpringContextHolder.getBean(TOeExamRecordService.class);
    private static RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);

    public static Long getId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "id");
    }

    public static Long getExamStudentId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "examStudentId");
    }

    public static Long getPaperId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "paperId");
    }

    public static void setFirstStartTime(Long recordId, Date date) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "firstStartTime", date);
        examRecordService.dataUpdateMq(recordId, "first_start_time", date, 1);
    }

    public static void setStatus(Long recordId, ExamRecordStatusEnum status) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "status", status);
        examRecordService.dataUpdateMq(recordId, "status", status);
    }

    public static Integer getDurationSeconds(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "durationSeconds");
    }

    public static Double getObjectiveScore(Long recordId) {
        return (Double) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "objectiveScore");
    }

    public static Long getExamId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "examId");
    }

    public static void setObjectiveScore(Long recordId, Double objectiveScore) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "objectiveScore", objectiveScore);
        examRecordService.dataUpdateMq(recordId, "objective_score", objectiveScore);
    }

    public static void setFinishTime(Long recordId, Date finishTime) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "finishTime", finishTime);
        examRecordService.dataUpdateMq(recordId, "finish_time", finishTime, 1);
    }

    public static Date getFinishTime(Long recordId) {
        return (Date) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "finishTime");
    }

    public static void setDurationSeconds(Long recordId, Integer durationSeconds) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "durationSeconds", durationSeconds);
        examRecordService.dataUpdateMq(recordId, "duration_seconds", durationSeconds);
    }

    public static void setFinishType(Long recordId, FinishTypeEnum finishType) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "finishType", finishType);
        examRecordService.dataUpdateMq(recordId, "finish_type", finishType);
    }

    public static FinishTypeEnum getFinishType(Long recordId) {
        return (FinishTypeEnum) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "finishType");
    }

    public static Date getClientLastSyncTime(Long recordId) {
        return (Date) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "clientLastSyncTime");
    }

    public static Integer getLeftBreakResumeCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "leftBreakResumeCount");
    }

    public static void setLeftBreakResumeCount(Long recordId, Integer leftBreakResumeCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "leftBreakResumeCount", leftBreakResumeCount);
        examRecordService.dataUpdateMq(recordId, "left_break_resume_count", leftBreakResumeCount);
    }

    public static ExamRecordStatusEnum getStatus(Long recordId) {
        return (ExamRecordStatusEnum) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "status");
    }

    public static Long getLastBreakId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "lastBreakId");
    }

    public static Date getLastBreakTime(Long recordId) {
        return (Date) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "lastBreakTime");
    }

    public static void setEntryAuthenticationResult(Long recordId, VerifyExceptionEnum entryAuthenticationResult) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "entryAuthenticationResult", entryAuthenticationResult);
        examRecordService.dataUpdateMq(recordId, "entry_authentication_result", entryAuthenticationResult);
    }

    public static void setEntryAuthenticationId(Long recordId, Long entryAuthenticationId) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "entryAuthenticationId", entryAuthenticationId);
        examRecordService.dataUpdateMq(recordId, "entry_authentication_id", entryAuthenticationId);
    }

    public static Long getExamActivityId(Long recordId) {
        return (Long) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "examActivityId");
    }

    public static void setWarningCount(Long recordId, Integer warningCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "warningCount", warningCount);
        examRecordService.dataUpdateMq(recordId, "warning_count", warningCount);
    }

    public static Integer getWarningCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "warningCount");
    }

    public static void setBreachStatus(Long recordId, Integer breachStatus) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "breachStatus", breachStatus);
        examRecordService.dataUpdateMq(recordId, "breach_status", breachStatus);
    }

    public static Integer getBreachStatus(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "breachStatus");
    }

    public static void setInProcessLivenessVerifyCount(Long recordId, Integer inProcessLivenessVerifyCount) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "inProcessLivenessVerifyCount", inProcessLivenessVerifyCount);
        examRecordService.dataUpdateMq(recordId, "in_process_liveness_verify_count", inProcessLivenessVerifyCount);
    }

    public static Integer getInProcessLivenessVerifyCount(Long recordId) {
        return (Integer) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "inProcessLivenessVerifyCount");
    }

    public static void setMonitorKey(Long recordId, String monitorKey) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "monitorKey", monitorKey);
        examRecordService.dataUpdateMq(recordId, "monitor_key", monitorKey);
    }

    public static String getMonitorKey(Long recordId) {
        if (Objects.isNull(redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "monitorKey"))) {
            setMonitorKey(recordId, String.valueOf(redisUtil.getRedisSequence()));
        }
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "monitorKey");
    }

    public static String getMonitorLiveUrl(Long recordId, String source) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + source);
    }

    public static String getMonitorLiveUrlMobileFirst(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_FIRST.name());
    }

    public static String getMonitorLiveUrlMobileSecond(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_SECOND.name());
    }

    public static String getMonitorLiveUrlClientCamera(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.CLIENT_CAMERA.name());
    }

    public static String getMonitorLiveUrlClientScreen(Long recordId) {
        return (String) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.CLIENT_SCREEN.name());
    }

    public static void setMonitorLiveUrl(Long recordId, String source, String liveUrl) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_LIVE_URL_ + source, liveUrl);
        examRecordService.dataUpdateMq(recordId, "monitor_live_url", liveUrl);
    }

    public static MonitorStatusSourceEnum getMonitorStatus(Long recordId, String source) {
        return (MonitorStatusSourceEnum) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_STATUS_ + source);
    }

    public static void setMonitorStatus(Long recordId, String source, MonitorStatusSourceEnum statusSourceEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_STATUS_ + source, statusSourceEnum);
        examRecordService.dataUpdateMq(recordId, "monitor_status_source", statusSourceEnum);
    }

    public static MonitorCallStatusSourceEnum getMonitorCallStatus(Long recordId, String source) {
        return (MonitorCallStatusSourceEnum) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_CALL_STATUS_ + source);
    }

    public static void setMonitorCallStatus(Long recordId, String source, MonitorCallStatusSourceEnum callStatusSourceEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), SystemConstant.MONITOR_CALL_STATUS_ + source, callStatusSourceEnum.name());
    }

    public static WebsocketStatusEnum getClientWebsocketStatus(Long recordId) {
        return (WebsocketStatusEnum) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "clientWebsocketStatus");
    }

    public static void setClientWebsocketStatus(Long recordId, WebsocketStatusEnum websocketStatusEnum) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "clientWebsocketStatus", websocketStatusEnum);
        examRecordService.dataUpdateMq(recordId, "client_websocket_status", websocketStatusEnum);
    }

    public static void setClientCurrentIp(Long recordId, String ip) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "clientCurrentIp", ip);
        examRecordService.dataUpdateMq(recordId, "client_current_ip", ip);
    }

    public static void setClientWebsocketId(Long recordId, String id) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "clientWebsocketId", id);
        examRecordService.dataUpdateMq(recordId, "client_websocket_id", id);
    }

    public static void setClientLastSyncTime(Long recordId) {
        Date now = new Date();
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "clientLastSyncTime", now);
        examRecordService.dataUpdateMq(recordId, "client_last_sync_time", now);
    }

    public static void setPaperDownload(Long recordId, Integer paperDownload) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "paperDownload", paperDownload);
        examRecordService.dataUpdateMq(recordId, "paper_download", paperDownload);
    }

    public static void setAnswerProgress(Long recordId, Double progress) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "answerProgress", progress);
        examRecordService.dataUpdateMq(recordId, "answer_progress", progress);
    }

    public static void setLastBreakId(Long recordId, Long breakId) {
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "lastBreakId", breakId);
        examRecordService.dataUpdateMq(recordId, "last_break_id", breakId);
    }

    public static void setLastBreakTime(Long recordId) {
        Date now = new Date();
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "lastBreakTime", now);
        examRecordService.dataUpdateMq(recordId, "last_break_time", now);
    }

    public static void setLastStartTime(Long recordId) {
        Date now = new Date();
        redisUtil.set(RedisKeyHelper.examRecordCacheKey(recordId), "lastStartTime", now);
        examRecordService.dataUpdateMq(recordId, "last_start_time", now);
    }

    public static Date getLastStartTime(Long recordId) {
        return (Date) redisUtil.get(RedisKeyHelper.examRecordCacheKey(recordId), "lastStartTime");
    }
}
