package com.qmth.themis.business.entity;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.qmth.themis.business.enums.*;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;
import java.util.Date;

/**
 * @Description: 考试记录
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@ApiModel(value = "t_oe_exam_record", description = "考试记录")
public class TOeExamRecord implements Serializable {
    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键")
    @TableId(value = "id")
    private Long id;

    @ApiModelProperty(value = "考试批次ID")
    @TableField(value = "exam_id")
    private Long examId;

    @ApiModelProperty(value = "考试场次ID")
    @TableField(value = "exam_activity_id")
    private Long examActivityId;

    @ApiModelProperty(value = "考生ID")
    @TableField(value = "exam_student_id")
    private Long examStudentId;

    @ApiModelProperty(value = "实际使用的试卷ID")
    @TableField(value = "paper_id")
    private Long paperId;

    @ApiModelProperty(value = "考试状态，first_prepare：首次候考，answering：正在答题，break_off：已中断，resume_prepare：断点恢复候考，finished：已结束考试，persisted：数据已保存")
    @TableField(value = "status")
    private ExamRecordStatusEnum status;

    @ApiModelProperty(value = "首次进入候考时间")
    @TableField(value = "first_prepare_time")
    private Date firstPrepareTime;

    @ApiModelProperty(value = "首次开考时间")
    @TableField(value = "first_start_time")
    private Date firstStartTime;

    @ApiModelProperty(value = "最近断点时间")
    @TableField(value = "last_break_time")
    private Date lastBreakTime;

    @ApiModelProperty(value = "最近恢复候考时间")
    @TableField(value = "last_prepare_time")
    private Date lastPrepareTime;

    @ApiModelProperty(value = "最近恢复开考时间")
    @TableField(value = "last_start_time")
    private Date lastStartTime;

    @ApiModelProperty(value = "剩余断点续考次数")
    @TableField(value = "left_break_resume_count")
    private Integer leftBreakResumeCount;

    @ApiModelProperty(value = "客户端当前IP地址")
    @TableField(value = "client_current_ip")
    private String clientCurrentIp;

    @ApiModelProperty(value = "客户端websocket状态")
    @TableField(value = "client_websocket_status")
    private WebsocketStatusEnum clientWebsocketStatus;

    @ApiModelProperty(value = "客户端websocket连接标识")
    @TableField(value = "client_websocket_id")
    private String clientWebsocketId;

    @ApiModelProperty(value = "客户端最近同步时间")
    @TableField(value = "client_last_sync_time")
    private Date clientLastSyncTime;

    @ApiModelProperty(value = "答题进度")
    @TableField(value = "answer_progress")
    private Double answerProgress;

    @ApiModelProperty(value = "累计考试用时")
    @TableField(value = "duration_seconds")
    private Integer durationSeconds;

    @ApiModelProperty(value = "交卷时间")
    @TableField(value = "finish_time")
    private Date finishTime;

    @ApiModelProperty(value = "交卷原因，hand：手动交卷，auto：系统交卷，breach：违纪交卷")
    @TableField(value = "finish_type")
    private FinishTypeEnum finishType;

    @ApiModelProperty(value = "预警次数")
    @TableField(value = "warning_count")
    private Integer warningCount;

    @ApiModelProperty(value = "审核结果，pass：通过，no_pass：不通过")
    @TableField(value = "review_result")
    private ReviewResultEnum reviewResult;

    @ApiModelProperty(value = "客观题分数")
    @TableField(value = "objective_score")
    private Double objectiveScore;

    @ApiModelProperty(value = "试题下载，0：是，1：不是")
    @TableField(value = "paper_download")
    private Integer paperDownload;

    @ApiModelProperty(value = "是否违纪，0：是，1：不是")
    @TableField(value = "breach_status")
    private Integer breachStatus;

    @ApiModelProperty(value = "个人试卷结构")
    @TableField(value = "paper_struct_path")
    private String paperStructPath;

    @ApiModelProperty(value = "个人试卷结构是否已上传，0：未上传，1：已上传")
    @TableField(value = "paper_struct_upload")
    private Integer paperStructUpload;

    @ApiModelProperty(value = "第几次考试")
    @TableField(value = "serial_number")
    private Integer serialNumber;

    @ApiModelProperty(value = "最近断点记录ID")
    @TableField(value = "last_break_id")
    private Long lastBreakId;

    @ApiModelProperty(value = "开考身份验证结果")
    @TableField(value = "entry_authentication_result")
    private VerifyExceptionEnum entryAuthenticationResult;

    @ApiModelProperty(value = "开考身份验证记录ID")
    @TableField(value = "entry_authentication_id")
    private Long entryAuthenticationId;

    @ApiModelProperty(value = "当前过程人脸检测状态")
    @TableField(value = "in_process_face_verify_status")
    private Integer inProcessFaceVerifyStatus;

    @ApiModelProperty(value = "当前过程活体验证状态")
    @TableField(value = "in_process_liveness_verify_status")
    private Integer inProcessLivenessVerifyStatus;

    @ApiModelProperty(value = "已完成过程活体验证次数")
    @TableField(value = "in_process_liveness_verify_count")
    private Integer inProcessLivenessVerifyCount;

    @ApiModelProperty(value = "监控标识（默认直接用recordId）")
    @TableField(value = "monitor_key")
    private String monitorKey;

    @ApiModelProperty(value = "监控状态，stop：停止，start：正常")
    @TableField(value = "monitor_status_source")
    private MonitorStatusSourceEnum monitorStatusSource;

    @ApiModelProperty(value = "监控观看地址")
    @TableField(value = "monitor_live_url")
    private String monitorLiveUrl;

    @ApiModelProperty(value = "算分状态，never：从未算分，calculating：正在算分，finish：算分完成")
    @TableField(value = "score_status")
    private ScoreStatusEnum scoreStatus;

    public ScoreStatusEnum getScoreStatus() {
        return scoreStatus;
    }

    public void setScoreStatus(ScoreStatusEnum scoreStatus) {
        this.scoreStatus = scoreStatus;
    }

    public String getMonitorKey() {
        return monitorKey;
    }

    public void setMonitorKey(String monitorKey) {
        this.monitorKey = monitorKey;
    }

    public MonitorStatusSourceEnum getMonitorStatusSource() {
        return monitorStatusSource;
    }

    public void setMonitorStatusSource(MonitorStatusSourceEnum monitorStatusSource) {
        this.monitorStatusSource = monitorStatusSource;
    }

    public String getMonitorLiveUrl() {
        return monitorLiveUrl;
    }

    public void setMonitorLiveUrl(String monitorLiveUrl) {
        this.monitorLiveUrl = monitorLiveUrl;
    }

    public static long getSerialVersionUID() {
        return serialVersionUID;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getExamId() {
        return examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public Long getExamActivityId() {
        return examActivityId;
    }

    public void setExamActivityId(Long examActivityId) {
        this.examActivityId = examActivityId;
    }

    public Long getExamStudentId() {
        return examStudentId;
    }

    public void setExamStudentId(Long examStudentId) {
        this.examStudentId = examStudentId;
    }

    public Long getPaperId() {
        return paperId;
    }

    public void setPaperId(Long paperId) {
        this.paperId = paperId;
    }

    public ExamRecordStatusEnum getStatus() {
        return status;
    }

    public void setStatus(ExamRecordStatusEnum status) {
        this.status = status;
    }

    public Date getFirstPrepareTime() {
        return firstPrepareTime;
    }

    public void setFirstPrepareTime(Date firstPrepareTime) {
        this.firstPrepareTime = firstPrepareTime;
    }

    public Date getFirstStartTime() {
        return firstStartTime;
    }

    public void setFirstStartTime(Date firstStartTime) {
        this.firstStartTime = firstStartTime;
    }

    public Date getLastBreakTime() {
        return lastBreakTime;
    }

    public void setLastBreakTime(Date lastBreakTime) {
        this.lastBreakTime = lastBreakTime;
    }

    public Date getLastPrepareTime() {
        return lastPrepareTime;
    }

    public void setLastPrepareTime(Date lastPrepareTime) {
        this.lastPrepareTime = lastPrepareTime;
    }

    public Date getLastStartTime() {
        return lastStartTime;
    }

    public void setLastStartTime(Date lastStartTime) {
        this.lastStartTime = lastStartTime;
    }

    public Integer getLeftBreakResumeCount() {
        return leftBreakResumeCount;
    }

    public void setLeftBreakResumeCount(Integer leftBreakResumeCount) {
        this.leftBreakResumeCount = leftBreakResumeCount;
    }

    public String getClientCurrentIp() {
        return clientCurrentIp;
    }

    public void setClientCurrentIp(String clientCurrentIp) {
        this.clientCurrentIp = clientCurrentIp;
    }

    public WebsocketStatusEnum getClientWebsocketStatus() {
        return clientWebsocketStatus;
    }

    public void setClientWebsocketStatus(WebsocketStatusEnum clientWebsocketStatus) {
        this.clientWebsocketStatus = clientWebsocketStatus;
    }

    public String getClientWebsocketId() {
        return clientWebsocketId;
    }

    public void setClientWebsocketId(String clientWebsocketId) {
        this.clientWebsocketId = clientWebsocketId;
    }

    public Date getClientLastSyncTime() {
        return clientLastSyncTime;
    }

    public void setClientLastSyncTime(Date clientLastSyncTime) {
        this.clientLastSyncTime = clientLastSyncTime;
    }

    public Double getAnswerProgress() {
        return answerProgress;
    }

    public void setAnswerProgress(Double answerProgress) {
        this.answerProgress = answerProgress;
    }

    public Integer getDurationSeconds() {
        return durationSeconds;
    }

    public void setDurationSeconds(Integer durationSeconds) {
        this.durationSeconds = durationSeconds;
    }

    public Date getFinishTime() {
        return finishTime;
    }

    public void setFinishTime(Date finishTime) {
        this.finishTime = finishTime;
    }

    public FinishTypeEnum getFinishType() {
        return finishType;
    }

    public void setFinishType(FinishTypeEnum finishType) {
        this.finishType = finishType;
    }

    public Integer getWarningCount() {
        return warningCount;
    }

    public void setWarningCount(Integer warningCount) {
        this.warningCount = warningCount;
    }

    public ReviewResultEnum getReviewResult() {
        return reviewResult;
    }

    public void setReviewResult(ReviewResultEnum reviewResult) {
        this.reviewResult = reviewResult;
    }

    public Double getObjectiveScore() {
        return objectiveScore;
    }

    public void setObjectiveScore(Double objectiveScore) {
        this.objectiveScore = objectiveScore;
    }

    public Integer getPaperDownload() {
        return paperDownload;
    }

    public void setPaperDownload(Integer paperDownload) {
        this.paperDownload = paperDownload;
    }

    public Integer getBreachStatus() {
        return breachStatus;
    }

    public void setBreachStatus(Integer breachStatus) {
        this.breachStatus = breachStatus;
    }

    public String getPaperStructPath() {
        return paperStructPath;
    }

    public void setPaperStructPath(String paperStructPath) {
        this.paperStructPath = paperStructPath;
    }

    public Integer getPaperStructUpload() {
        return paperStructUpload;
    }

    public void setPaperStructUpload(Integer paperStructUpload) {
        this.paperStructUpload = paperStructUpload;
    }

    public Integer getSerialNumber() {
        return serialNumber;
    }

    public void setSerialNumber(Integer serialNumber) {
        this.serialNumber = serialNumber;
    }

    public Long getLastBreakId() {
        return lastBreakId;
    }

    public void setLastBreakId(Long lastBreakId) {
        this.lastBreakId = lastBreakId;
    }

    public VerifyExceptionEnum getEntryAuthenticationResult() {
        return entryAuthenticationResult;
    }

    public void setEntryAuthenticationResult(VerifyExceptionEnum entryAuthenticationResult) {
        this.entryAuthenticationResult = entryAuthenticationResult;
    }

    public Long getEntryAuthenticationId() {
        return entryAuthenticationId;
    }

    public void setEntryAuthenticationId(Long entryAuthenticationId) {
        this.entryAuthenticationId = entryAuthenticationId;
    }

    public Integer getInProcessFaceVerifyStatus() {
        return inProcessFaceVerifyStatus;
    }

    public void setInProcessFaceVerifyStatus(Integer inProcessFaceVerifyStatus) {
        this.inProcessFaceVerifyStatus = inProcessFaceVerifyStatus;
    }

    public Integer getInProcessLivenessVerifyStatus() {
        return inProcessLivenessVerifyStatus;
    }

    public void setInProcessLivenessVerifyStatus(Integer inProcessLivenessVerifyStatus) {
        this.inProcessLivenessVerifyStatus = inProcessLivenessVerifyStatus;
    }

    public Integer getInProcessLivenessVerifyCount() {
        return inProcessLivenessVerifyCount;
    }

    public void setInProcessLivenessVerifyCount(Integer inProcessLivenessVerifyCount) {
        this.inProcessLivenessVerifyCount = inProcessLivenessVerifyCount;
    }
}
