package com.qmth.themis.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.dto.WarningDto;
import com.qmth.themis.business.entity.TEConfig;
import com.qmth.themis.business.entity.TIeInvigilateWarnInfo;
import com.qmth.themis.business.entity.TOeFaceVerifyHistory;
import com.qmth.themis.business.enums.VerifyExceptionEnum;
import com.qmth.themis.business.enums.WarningLevelEnum;
import com.qmth.themis.business.service.TEConfigService;
import com.qmth.themis.business.service.TIeInvigilateWarnInfoService;
import com.qmth.themis.business.service.TOeFaceVerifyHistoryService;
import com.qmth.themis.business.service.WarningService;
import com.qmth.themis.business.util.RedisUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: 预警规则公用
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/8/18
 */
@Service
public class WarningServiceImpl implements WarningService {
    private final static Logger log = LoggerFactory.getLogger(WarningServiceImpl.class);

    @Resource
    TEConfigService teConfigService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    TOeFaceVerifyHistoryService faceVerifyHistoryService;

    @Resource
    TIeInvigilateWarnInfoService tIeInvigilateWarnInfoService;

    /**
     * 人脸数量异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void faceCountError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        Integer faceCount = warningDto.getFaceCount();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        if (faceCount > 1) {//多张人脸
            Integer count = faceVerifyHistoryService.faceCountError(recordId, warningDto.getWarningEnum().name(), faceCount, true);
            count = Objects.isNull(count) ? 0 : count;
            if (count >= teConfig.getMultipleFaceCountError()) {
                TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(1), WarningLevelEnum.valueOf(warningEnum.getLevel().get(1)).getDesc(), warningEnum, photoUrl);
                tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                this.setWarningCount(recordId);
            }
        } else if (faceCount <= 0) {//未检测到人脸
            Integer count = faceVerifyHistoryService.faceCountError(recordId, warningDto.getWarningEnum().name(), faceCount, false);
            count = Objects.isNull(count) ? 0 : count;
            if (count >= teConfig.getNoFaceCountError()) {
                TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(0), WarningLevelEnum.valueOf(warningEnum.getLevel().get(0)).getDesc(), warningEnum, photoUrl);
                tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
                this.setWarningCount(recordId);
            }
        }
    }

    /**
     * 人脸比对异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void faceCompareError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        String photoUrl = warningDto.getPhotoUrl();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        Integer count = faceVerifyHistoryService.faceCompareError(recordId, warningDto.getWarningEnum().name());
        count = Objects.isNull(count) ? 0 : count;
        if (count >= teConfig.getMatchFaceCompareErrorCount()) {
            TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(0), WarningLevelEnum.valueOf(warningEnum.getLevel().get(0)).getDesc(), warningEnum, photoUrl);
            tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
            this.setWarningCount(recordId);
        }
        if (count >= teConfig.getTotalFaceCompareErrorCount()) {
            TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, warningEnum.getLevel().get(1), WarningLevelEnum.valueOf(warningEnum.getLevel().get(1)).getDesc(), warningEnum, photoUrl);
            tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
            this.setWarningCount(recordId);
        }
    }

    /**
     * 闭眼检测异常
     *
     * @param warningDto
     */
    @Override
    public void eyeCloseError(WarningDto warningDto) {

    }

    /**
     * 活体动作异常
     *
     * @param warningDto
     */
    @Override
    public void livenessActionError(WarningDto warningDto) {

    }

    /**
     * 真实性异常
     *
     * @param warningDto
     */
    @Override
    @Transactional
    public void realnessError(WarningDto warningDto) {
        Long recordId = warningDto.getRecordId();
        VerifyExceptionEnum warningEnum = warningDto.getWarningEnum();
        TEConfig teConfig = teConfigService.getGlobalConfig();
        Long examId = ExamRecordCacheUtil.getExamId(recordId);
        Long examStudentId = ExamRecordCacheUtil.getExamStudentId(recordId);
        Long examActivityId = ExamRecordCacheUtil.getExamActivityId(recordId);
        QueryWrapper<TOeFaceVerifyHistory> tOeFaceVerifyHistoryQueryWrapper = new QueryWrapper<>();
        tOeFaceVerifyHistoryQueryWrapper.lambda().eq(TOeFaceVerifyHistory::getExamRecordId, recordId).eq(TOeFaceVerifyHistory::getException, warningEnum.name());
        int count = faceVerifyHistoryService.count(tOeFaceVerifyHistoryQueryWrapper);
        if (count > teConfig.getRealnessCount()) {
            TIeInvigilateWarnInfo tIeInvigilateWarnInfo = new TIeInvigilateWarnInfo(examId, examActivityId, recordId, examStudentId, WarningLevelEnum.D15.name(), WarningLevelEnum.D15.getDesc(), warningEnum);
            tIeInvigilateWarnInfoService.saveOrUpdate(tIeInvigilateWarnInfo);
            this.setWarningCount(recordId);
        }
    }

    /**
     * 设置预警数
     *
     * @param recordId
     */
    public void setWarningCount(Long recordId) {
        Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        if (Objects.nonNull(objectMap.get("warningCount"))) {
            Integer warningCount = ExamRecordCacheUtil.getWarningCount(recordId);
            warningCount++;
            ExamRecordCacheUtil.setWarningCount(recordId, warningCount);
        } else {
            ExamRecordCacheUtil.setWarningCount(recordId, 1);
        }
    }
}
